/*======================================================================================

	Program:		Range Imaging Toolkit (RITK)
	Module:			ritkVisualizationWidget2D.h
	Version:		$Revision: 1821 $
	Date:			$Date: 2011-10-02 15:41:17 +0200 (Sun, 02 Oct 2011) $

	Copyright (c) 2010-2011
	Pattern Recognition Lab, 
    University Erlangen-Nuremberg, Germany
    All rights reserved.

	This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
	See RITKCopyright.txt for more information.

	http://www5.informatik.uni-erlangen.de/research/software/ritk

========================================================================================*/

#ifndef VISUALIZATIONWIDGET2D_H__
#define VISUALIZATIONWIDGET2D_H__

#include "ritkVTK.h"

#include <QWidget>
#include <QMutex>
#include "ui_ritkVisualizationWidget2D.h"

#include "ritkRImage2DActorPipeline.h"
#include "ritkVTKImageActorPipeline.h"

#include <vtkSmartPointer.h>
#include <vtkCamera.h>
#include <vtkImageActor.h>
#include <vtkRenderer.h>
#include <vtkRenderWindow.h>
#include <vtkPolyData.h>
#include <vtkPolyDataMapper.h>
#include <vtkDelaunay2D.h>
#include <vtkCellArray.h>
#include <vtkInteractorStyleImage.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkstd/vector>

#include <QVTKWidget.h>

Q_DECLARE_METATYPE(vtkSmartPointer<ritk::VTKImageActorPipeline>)

namespace ritk
{


/**	@ingroup	ritkVTK
 *	@class		VisualizationWidget2D
 *	@author		Sebastian Bauer
 *	@brief		A visualization widget
 *
 *	@details
 *	A widget used to visualize 2D data.
 *	@todo	Needs a detailed documentation on how actors/props are manged asap. Also document thread safety.
 */
class RITKVTK_EXPORT VisualizationWidget2D : public QWidget, public Ui_VisualizationWidget2D
{
	Q_OBJECT

public:
	/// Constructor
	VisualizationWidget2D(QWidget *parent=0);
	/// Destructor
	~VisualizationWidget2D();
	
	/// Get the vtk render window
	vtkRenderWindow* GetRenderWindow();

public slots:
	/// UpdateGUI. Note that this method has to be called from the main thread context
	void UpdateGUI();
	/// Add an actor 
	void AddActor(vtkSmartPointer<vtkImageActor> Actor);	
	/// Remove an actor 
	void RemoveActor(vtkSmartPointer<vtkImageActor> Actor);
	/// Add a prop
	void AddProp3D(vtkSmartPointer<vtkProp3D> Prop3D);	
	/// Remove a prop
	void RemoveProp3D(vtkSmartPointer<vtkProp3D> Prop3D);

protected:
	/// The renderer
	vtkSmartPointer<vtkRenderer> m_Renderer;
	/// The render window
	vtkSmartPointer<vtkRenderWindow> m_RenderWindow;

	/// VTK camera
	vtkSmartPointer<vtkCamera> m_Camera;
	/// Flag that indicates whether the camera has been initialized
	bool m_CameraInitFlag;

	/// The current 3D props
	std::vector<vtkSmartPointer<vtkProp3D> > m_Prop3Ds;

	/// The QVTKWidget interactor style
	vtkSmartPointer<vtkInteractorStyleImage> m_InteractorStyleImage;
	/// The QVTKWidget interactor (vtkInteractor does not work for QVTKWidget)
	vtkSmartPointer<QVTKInteractor> m_Interactor;
};

} // end namespace ritk

#endif // VisualizationWidget2D_H__
